
#include <stdio.h>
#include <stdint.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>

#define BAUD 38400UL

#define MYUBRR (((F_CPU + BAUD * 8LU) / (BAUD * 16LU) - 1LU))

static FILE mystdout = { 0 };

volatile unsigned long t1_ticks = 0;

volatile unsigned char flag = 0;

/********************************************************************************/

static int uart_putchar (char c, FILE *stream)
	{
	while ( !( UCSR0A & (1<<UDRE0)) )
		;

	UDR0 = c;

	return 0;
	}

/********************************************************************************/

void uart_init (void)
	{
	DDRD |= 0b10000000;

	UBRR0H = (unsigned char) (MYUBRR >> 8);

	UBRR0L = (unsigned char) (MYUBRR & 0xFF);

	UCSR0B = (1<<RXEN0) | (1<<TXEN0);

	UCSR0C = (3 << UCSZ00);

	fdev_setup_stream (&mystdout, uart_putchar, NULL, _FDEV_SETUP_WRITE);

	stdout = &mystdout;
	}

/********************************************************************************/

ISR (TIMER1_OVF_vect)
	{
	t1_ticks += 0x10000;
	}

/********************************************************************************/

ISR (TIMER1_CAPT_vect)
	{
	unsigned char sreg = SREG;

	unsigned int icr1;

	cli ();

	icr1 = ICR1;

	if ((TIFR1 & (1<<TOV1)) && (icr1 < 0x8000))
		t1_ticks += 0x1000;

	t1_ticks += (unsigned long) icr1;

	flag = 1;

	SREG = sreg;
	}

/********************************************************************************/

void t1_init (void)
	{
	/** Input capture Pin ICP1 (Pin 14) = Eingang **/

	DDRB &= 0b11111110;

	/** PRR – Power Reduction Register (S. 54) **/

	PRR &= 0b11110111;

	/** TCCR1A – Timer/Counter1 Control Register A (S. 140) **/

	TCCR1A = 0;

	/** TCCR1B – Timer/Counter1 Control Register B (S. 142) **/

	TCCR1B = 0b11000001;

	/** TIMSK1 – Timer/Counter1 Interrupt Mask Register (S. 144) **/

	TIMSK1 = 0b00100001;

	/*** TIFR1 – Timer/Counter1 Interrupt Flag Register (S. 145) **/

	TIFR1 &= 0b00100111;

	/** Interrupts global ein **/

	sei ();

	}

/********************************************************************************/

int main (void)
	{
	uart_init ();

	t1_init ();

	while (1)
		{
		while (flag == 0)
			;

		printf ("%lu\r\n", t1_ticks);

		t1_ticks = 0;

		flag = 0;
		}

	return 0;
	}
